//	Initialize the copyWidget object
prism.run([function() {

	var supportedChartTypes = ["chart/column","chart/bar"];

	var comparisonIndex = 0;
	
	var defaultSettings = {
		showVariance : false
	}	
	
	//	Function to determine if this chart type is supported
	function widgetIsSupported(type){
		if (supportedChartTypes.indexOf(type) >= 0) {
			return true;
		} else {
			return false;
		}
	}

	//	Function to show a buttefly Chart
	function overrideValueLabels(w) {

		//	Should this function be run?
		var shouldRun = (w.options.overrideValueLabels) && (w.options.overrideValueLabels.showVariance);
		if (!shouldRun) {
			return;
		}

		//	Make sure the widget has quadrantAnalysis options defined
		var settings = w.options.overrideValueLabels;
		if (typeof  settings === "undefined") {
			//	Define the widget's settings
			settings = defaultSettings;
		}

		//	Get the chart
		var chart = w.queryResult;

		//	Get the data series
		var series = chart.series;

		//	Loop through data series
		for (var i=0; i<series.length; i++){

			//	Skip if this series is the comparison index
			if (i !== comparisonIndex) {

				//	Loop through each data point
				for (var j=0; j<series[i].data.length; j++) {

					//	Get the comparison value
					var comparisonValue = series[comparisonIndex].data[j].y

					//	Calculate the variance 
					series[i].data[j].variance = series[i].data[j].y - comparisonValue;					
				}
			}
		}

		//	Define custom formatter for value labels
		var valueLabelFormatter = function () {
			
			//	Get the formatter
			var func=$$get(this,"series.options.mask");

			//	Look for the variance value
			var variance = this.point.options.variance;

			//	Figure out what to render
			var valueText = "";
			if (typeof variance !== "undefined") {
				//	Format the value
				value = defined(func) ? func(variance) : formatWithCommas(variance);
				if (variance>0) {
					//	Handle positive variances
					valueText = "+" + value;
				} else if (variance<0) {
					//	Handle negative variances
					valueText = "(" + value.replace("-","") + ")";
				} else {
					//	Nothing special needed for zero values
					valueText = "0";
				}

			}
			
			//	Return the formatted value
			return valueText;
		}

		//	Set the custom formatter
		chart.plotOptions.series.dataLabels.formatter = valueLabelFormatter;
		chart.plotOptions.series.dataLabels.y = -2;		  	
	}
	
	// Registering dashboard/ widget creation in order to perform drilling
	prism.on("dashboardloaded", function (e, args) {
		args.dashboard.on("widgetinitialized", onWidgetInitialized);
	});

	// register widget events upon initialization
	function onWidgetInitialized(dashboard, args) {
		//	Hooking to ready/destroyed events
		args.widget.on("destroyed", onWidgetDestroyed);
		//	Should we run a quadrant analysis?
		var shouldInit = widgetIsSupported(args.widget.type);
		//	Add hook for the quadrant analysis
		if (shouldInit) {
			args.widget.on("render", overrideValueLabels);
		}
	}

	// unregistering widget events
	function onWidgetDestroyed(widget, args) {
		widget.off("destroyed", onWidgetDestroyed);
	}

	//	Create Options for the quadrant analysis
	prism.on("beforemenu", function (e, args) {
		try {
			//	Define the menu label
			var headerMenuCaption = "Value Labels";

			//	Has the menu been added already?
			var addMenuItem = true;
			$.each(args.settings.items, function(){
				if (this.caption === headerMenuCaption) {
					addMenuItem = false;
				}
			})

			//	Only show this in the widget editor
			var widgetEditorOpen = (prism.$ngscope.appstate == "widget");
			if (!widgetEditorOpen) {
				addMenuItem = false;
				return;
			}
			
			// find the widget
			var widget = e.currentScope.widget;
			if(widget == undefined){
				addMenuItem = false;
				return;
			}	

			// Widget must be a bar chart
			if(!widgetIsSupported(widget.type)){
				addMenuItem = false;
				return;
			}

			//	Make sure the user clicked on a widget menu
			var target = args.ui.target;
			if (target.attr('title') !== 'Options'){
				addMenuItem = false;
			}

			//	Add the option to enable the quadrant analysis
			if (addMenuItem) {

				//	Get Saved Quadrant Analysis settings
				var settings = widget.options.overrideValueLabels;
				if (typeof settings === 'undefined'){
					//	Create defaults, if not defined
					settings = defaultSettings;
				}
				
				//	Function to handle events
				var menuItemClicked = function(){
					//	Update the settings				
					switch(this.caption){
						case 'Show Variance': 
							this.settings.showVariance = !this.settings.showVariance;
							break;		
					}
					//	Save to widget
					widget.options.overrideValueLabels = this.settings;
					//	Redraw the widget
					widget.redraw();
				}

				//	Create header
				var header = {
					caption: headerMenuCaption,
					type: "header",
					disabled: false
				};

				//	Create Enable option
				var enabled = {
					caption: "Show Variance",
					checked: settings.showVariance,
					closing: false,
					disabled: false,
					size: "xl",
					type: "check",
					widget : widget,
					settings : settings,
					execute: menuItemClicked
				};

				
				//	Add options to the menu
				args.settings.items.push(header);
				args.settings.items.push(enabled);	
			}

		}
		catch (e) {
			console.log(e);
		}
	});

}]);